#include <stdio.h>
#include <signal.h>
#include <time.h>

#include "open62541.h"

// bla

UA_Logger logger = Logger_Stdout;


UA_Boolean running = true;
static void stopHandler(int signal) {
    running = false;
}

static UA_StatusCode
readInteger(void *handle, const UA_NodeId nodeid, UA_Boolean sourceTimeStamp,
            const UA_NumericRange *range, UA_DataValue *dataValue) {
    dataValue->hasValue = true;

    UA_UInt32 t = time(0);   
    UA_Variant_setScalarCopy(&dataValue->value, &t, &UA_TYPES[UA_TYPES_INT32]);
    // we know the nodeid is a string
    UA_LOG_INFO(logger, UA_LOGCATEGORY_USERLAND, "Node read %.*s",
                nodeid.identifier.string.length, nodeid.identifier.string.data);
    UA_LOG_INFO(logger, UA_LOGCATEGORY_USERLAND, "read value %i", t);
    return UA_STATUSCODE_GOOD;
}

static UA_StatusCode
writeInteger(void *handle, const UA_NodeId nodeid,
             const UA_Variant *data, const UA_NumericRange *range) {
    if(UA_Variant_isScalar(data) && data->type == &UA_TYPES[UA_TYPES_INT32] && data->data){
        *(UA_UInt32*)handle = *(UA_UInt32*)data->data;
    }
    // we know the nodeid is a string
    UA_LOG_INFO(logger, UA_LOGCATEGORY_USERLAND, "Node written %.*s",
                nodeid.identifier.string.length, nodeid.identifier.string.data);
    UA_LOG_INFO(logger, UA_LOGCATEGORY_USERLAND, "written value %i", *(UA_UInt32*)handle);
    return UA_STATUSCODE_GOOD;
}

int main(void) {
    signal(SIGINT,  stopHandler);
    signal(SIGTERM, stopHandler);

    UA_ServerConfig config = UA_ServerConfig_standard;
    UA_ServerNetworkLayer nl = UA_ServerNetworkLayerTCP(UA_ConnectionConfig_standard, 16664);
    config.logger = Logger_Stdout;
    config.networkLayers = &nl;
    config.networkLayersSize = 1;
    UA_Server *server = UA_Server_new(config);

    UA_NodeId myObjectsId;
    UA_ObjectAttributes attr2;
    UA_ObjectAttributes_init(&attr2);
    attr2.description = UA_LOCALIZEDTEXT("en_US", "MyObjects");
    attr2.displayName = UA_LOCALIZEDTEXT("en_US", "A folder containing example objects and variables created by the client.");
    UA_Server_addObjectNode(
	   server, UA_NODEID_NUMERIC(1,1000),
	   UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER), UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES), 
	   UA_QUALIFIEDNAME(1, "MyObjects"),
	   UA_NODEID_NUMERIC(0, UA_NS0ID_FOLDERTYPE),
	   attr2, 0, &myObjectsId);

    /* add a variable node to the address space */
    UA_Int32 myInteger = 42;
    UA_NodeId myIntegerNodeId = UA_NODEID_STRING(1, "seconds.since.epoch");
    UA_QualifiedName myIntegerName = UA_QUALIFIEDNAME(1, "seconds since epoch");
    UA_DataSource dateDataSource = (UA_DataSource) {
        .handle = &myInteger, .read = readInteger, .write = writeInteger};
    UA_VariableAttributes attr;
    UA_VariableAttributes_init(&attr);
    attr.description = UA_LOCALIZEDTEXT("en_US","seconds since epoch");
    attr.displayName = UA_LOCALIZEDTEXT("en_US","seconds since epoch");


    UA_Server_addDataSourceVariableNode(server, myIntegerNodeId,
                                        myObjectsId,
                                        UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
                                        myIntegerName, UA_NODEID_NULL, attr, dateDataSource, NULL);


    UA_StatusCode retval = UA_Server_run(server, &running);
    UA_Server_delete(server);
    nl.deleteMembers(&nl);
    return retval;
}


